@@ -9,61 +9,14 @@
 # * A list of all email addresses found in mailto links
 # * A list of usernames/author details from meta data found in any documents on the site
 #
-# == Usage
-#
-# cewl [OPTION] ... URL
-#
-# -h, --help:
-#	 show help
-#
-# --depth x, -d x:
-#	 depth to spider to, default 2
-#
-# --min_word_length, -m:
-#	 minimum word length, default 3
-#
-# --email file, -e
-# --email_file file:
-#	 include any email addresses found during the spider, email_file is optional output file, if
-#	 not included the output is added to default output
-#
-# --meta file, -a
-# --meta_file file:
-#	 include any meta data found during the spider, meta_file is optional output file, if
-#	 not included the output is added to default output
-#
-# --no-words, -n
-#	 don't output the wordlist
-#
-# --offsite, -o:
-#	 let the spider visit other sites
-#
-# --write, -w file:
-#	 write the words to the file
-#
-# --ua, -u user-agent:
-#	 useragent to send
-#
-# --meta-temp-dir directory:
-#	 the temporary directory used by exiftool when parsing files, default /tmp
-#
-# --keep, -k:
-#	 keep the documents that are downloaded
-#
-# --count, -c:
-#	 show the count for each of the words found
-#
-# -v
-#	 verbose
-#
 # URL: The site to spider.
 #
 # Author:: Robin Wood (robin@digi.ninja)
-# Copyright:: Copyright (c) Robin Wood 2016
+# Copyright:: Copyright (c) Robin Wood 2018
 # Licence:: CC-BY-SA 2.0 or GPL-3+
 #
 
-VERSION = "5.3 (Heading Upwards)"
+VERSION = "5.4.3 (Arkanoid)"
 
 puts "CeWL #{VERSION} Robin Wood (robin@digi.ninja) (https://digi.ninja/)\n"
 
@@ -77,7 +30,7 @@ rescue LoadError => e
 	if e.to_s =~ /cannot load such file -- (.*)/
 		missing_gem = $1
 		puts "\nError: #{missing_gem} gem not installed\n"
-		puts "\t Use: 'gem install #{missing_gem}' to install the required gem\n\n"
+		puts "    Run 'bundle install' to install all the required gems\n\n"
 		exit 2
 	else
 		puts "There was an error loading the gems:\n"
@@ -86,10 +39,6 @@ rescue LoadError => e
 	end
 end
 
-# Doesn't work for some reason, maybe
-# because its bouncing into different classes
-trap("SIGINT") { throw :ctrl_c }
-
 require './cewl_lib'
 
 # Doing this so I can override the allowed? function which normally checks
@@ -183,15 +132,30 @@ class MySpiderInstance<SpiderInstance
 	attr_writer :verbose
 	attr_writer :debug
 
+	attr_writer :interrupt
+
 	# Force all files to be allowed
 	# Normally the robots.txt file will be honoured
 	def allowed?(a_url, parsed_url)
 		true
 	end
 
+	# Lifted from the original gem to fix the case statement
+	# which checked for Fixednum not Integer as
+	# Fixednum has been deprecated.
+	# 
+	def on(code, p = nil, &block)
+		f = p ? p : block
+		case code
+		when Integer
+			@callbacks[code] = f
+		else
+			@callbacks[code.to_sym] = f
+		end
+	end
+
 	def start! #:nodoc:
-		interrupted = false
-		trap("SIGINT") { interrupted = true }
+		trap("SIGINT") { puts 'Hold on, about to stop ...'; @interrupt = true }
 		begin
 			next_urls = @next_urls.pop
 			#tmp_n_u = {}
@@ -221,7 +185,7 @@ class MySpiderInstance<SpiderInstance
 					end
 
 					@teardown.call(a_url) unless @teardown.nil?
-					exit if interrupted
+					throw :ctrl_c if @interrupt
 				end
 			end
 		end while !@next_urls.empty?
@@ -230,6 +194,7 @@ class MySpiderInstance<SpiderInstance
 	def get_page(uri, &block) #:nodoc:
 		@seen << uri
 
+		trap("SIGINT") { puts 'Hold on, stopping here ...'; @interrupt = true }
 		begin
 			if @proxy_host.nil?
 				http = Net::HTTP.new(uri.host, uri.port)
@@ -279,7 +244,7 @@ class MySpiderInstance<SpiderInstance
 			end
 
 			res = http.request(req)
-
+			
 			if res.redirect?
 				puts "Redirect URL" if @debug
 				base_url = uri.to_s[0, uri.to_s.rindex('/')]
@@ -294,12 +259,15 @@ class MySpiderInstance<SpiderInstance
 			else
 				block.call(res)
 			end
+		rescue Zlib::DataError => e
+			puts "Error in Zlib decompressing data on #{uri}, moving on regardless"
 		rescue SocketError, Errno::EHOSTUNREACH => e
 			puts "Couldn't hit the site #{uri}, moving on"
 		rescue NoMethodError => e
 			if @verbose
 				puts "Unable to process URL"
 				puts "Message is #{e.to_s}"
+				puts e.backtrace
 			end
 		rescue => e
 			puts "\nUnable to connect to the site (#{uri.scheme}://#{uri.host}:#{uri.port}#{uri.request_uri})"
@@ -323,34 +291,41 @@ class MySpiderInstance<SpiderInstance
 		return nil if additional_url =~ /^#/
 
 		parsed_additional_url ||= URI.parse(additional_url)
-		case parsed_additional_url.scheme
-			when nil
-				u = base_url.is_a?(URI) ? base_url : URI.parse(base_url)
-				if additional_url[0].chr == '/'
-					"#{u.scheme}://#{u.host}#{additional_url}"
-				elsif u.path.nil? || u.path == ''
-					"#{u.scheme}://#{u.host}/#{additional_url}"
-				elsif u.path[0].chr == '/'
-					"#{u.scheme}://#{u.host}#{u.path}/#{additional_url}"
-				else
-					"#{u.scheme}://#{u.host}/#{u.path}/#{additional_url}"
-				end
+		if parsed_additional_url.scheme.nil?
+			u = base_url.is_a?(URI) ? base_url : URI.parse(base_url)
+			if additional_url[0].chr == '/'
+				url = "#{u.scheme}://#{u.host}:#{u.port}#{additional_url}"
+			elsif u.path.nil? || u.path == ''
+				url = "#{u.scheme}://#{u.host}:#{u.port}/#{additional_url}"
+			elsif u.path[0].chr == '/'
+				url = "#{u.scheme}://#{u.host}:#{u.port}#{u.path}/#{additional_url}"
 			else
-				additional_url
+				url = "#{u.scheme}://#{u.host}:#{u.port}/#{u.path}/#{additional_url}"
+			end
+		else
+			url = additional_url
 		end
+		return url
 	end
 
 	# Overriding the original spider one as it doesn't find hrefs very well
 	def generate_next_urls(a_url, resp) #:nodoc:
+		if @debug
+			puts "a_url = #{a_url}"
+			puts "resp = #{resp}"
+		end
 		web_page = resp.body
 		if URI.parse(a_url).path.empty?
 			base_url = a_url
 		else
 			base_url = a_url[0, a_url.rindex('/')]
 		end
+		puts "base_url: #{base_url}" if @debug
 
 		doc = Nokogiri::HTML(web_page)
 		links = doc.css('a').map { |a| a['href'] }
+
+		puts "links = #{links.inspect}" if @debug
 		links.map do |link|
 			begin
 				if link.nil?
@@ -403,7 +378,8 @@ end
 # A tree structure
 class Tree
 	attr :data
-	@max_depth
+	attr_writer :debug
+	attr_writer :max_depth
 	@children
 
 	# Get the maximum depth the tree can grow to
@@ -429,7 +405,7 @@ class Tree
 	end
 
 	# The constructor
-	def initialize(key=nil, value=nil, depth=0)
+	def initialize(key=nil, value=nil, depth=0, debug=false)
 		@data = TreeNode.new(key, value, depth)
 		@children = []
 		@max_depth = 2
@@ -461,6 +437,7 @@ class Tree
 
 	# Push an item onto the tree
 	def push(value)
+		puts "Pushing #{value}" if @debug
 		key = value.keys.first
 		value = value.values_at(key).first
 
@@ -470,12 +447,12 @@ class Tree
 			# If the depth is 0 then don't add anything to the tree
 			return if @max_depth == 0
 			if key == @data.value
-				child = Tree.new(key, value, @data.depth + 1)
+				child = Tree.new(key, value, @data.depth + 1, @debug)
 				@children << child
 			else
 				@children.each { |node|
 					if node.data.value == key && node.data.depth<@max_depth
-						child = Tree.new(key, value, node.data.depth + 1)
+						child = Tree.new(key, value, node.data.depth + 1, @debug)
 						@children << child
 					end
 				}
@@ -487,7 +464,7 @@ end
 opts = GetoptLong.new(
 		['--help', '-h', GetoptLong::NO_ARGUMENT],
 		['--keep', '-k', GetoptLong::NO_ARGUMENT],
-		['--depth', '-d', GetoptLong::OPTIONAL_ARGUMENT],
+		['--depth', '-d', GetoptLong::REQUIRED_ARGUMENT],
 		['--min_word_length', "-m", GetoptLong::REQUIRED_ARGUMENT],
 		['--no-words', "-n", GetoptLong::NO_ARGUMENT],
 		['--offsite', "-o", GetoptLong::NO_ARGUMENT],
@@ -496,6 +473,7 @@ opts = GetoptLong.new(
 		['--meta-temp-dir', GetoptLong::REQUIRED_ARGUMENT],
 		['--meta_file', GetoptLong::REQUIRED_ARGUMENT],
 		['--email_file', GetoptLong::REQUIRED_ARGUMENT],
+		['--with-numbers', GetoptLong::NO_ARGUMENT],
 		['--meta', "-a", GetoptLong::NO_ARGUMENT],
 		['--email', "-e", GetoptLong::NO_ARGUMENT],
 		['--count', '-c', GetoptLong::NO_ARGUMENT],
@@ -507,44 +485,48 @@ opts = GetoptLong.new(
 		['--proxy_port', GetoptLong::REQUIRED_ARGUMENT],
 		['--proxy_username', GetoptLong::REQUIRED_ARGUMENT],
 		['--proxy_password', GetoptLong::REQUIRED_ARGUMENT],
-		["--verbose", "-v", GetoptLong::NO_ARGUMENT]
+		["--verbose", "-v", GetoptLong::NO_ARGUMENT],
+		["--debug", GetoptLong::NO_ARGUMENT]
 )
 
 # Display the usage
 def usage
-	puts "Usage: cewl [OPTION] ... URL
-	--help, -h: show help
-	--keep, -k: keep the downloaded file
-	--depth x, -d x: depth to spider to, default 2
-	--min_word_length, -m: minimum word length, default 3
-	--offsite, -o: let the spider visit other sites
-	--write, -w file: write the output to the file
-	--ua, -u user-agent: user agent to send
-	--no-words, -n: don't output the wordlist
-	--meta, -a include meta data
-	--meta_file file: output file for meta data
-	--email, -e include email addresses
-	--email_file file: output file for email addresses
-	--meta-temp-dir directory: the temporary directory used by exiftool when parsing files, default /tmp
-	--count, -c: show the count for each word found
+	puts "Usage: cewl [OPTIONS] ... <url>
 
+    OPTIONS:
+	-h, --help: Show help.
+	-k, --keep: Keep the downloaded file.
+	-d <x>,--depth <x>: Depth to spider to, default 2.
+	-m, --min_word_length: Minimum word length, default 3.
+	-o, --offsite: Let the spider visit other sites.
+	-w, --write: Write the output to the file.
+	-u, --ua <agent>: User agent to send.
+	-n, --no-words: Don't output the wordlist.
+	--with-numbers: Accept words with numbers in as well as just letters
+	-a, --meta: include meta data.
+	--meta_file file: Output file for meta data.
+	-e, --email: Include email addresses.
+	--email_file <file>: Output file for email addresses.
+	--meta-temp-dir <dir>: The temporary directory used by exiftool when parsing files, default /tmp.
+	-c, --count: Show the count for each word found.
+	-v, --verbose: Verbose.
+	--debug: Extra debug information.
+      
 	Authentication
-		--auth_type: digest or basic
-		--auth_user: authentication username
-		--auth_pass: authentication password
-
+	--auth_type: Digest or basic.
+	--auth_user: Authentication username.
+	--auth_pass: Authentication password.
+      
 	Proxy Support
-		--proxy_host: proxy host
-		--proxy_port: proxy port, default 8080
-		--proxy_username: username for proxy, if required
-		--proxy_password: password for proxy, if required
-
+	--proxy_host: Proxy host.
+	--proxy_port: Proxy port, default 8080.
+	--proxy_username: Username for proxy, if required.
+	--proxy_password: Password for proxy, if required.
+      
 	Headers
-		--header, -H: in format name:value - can pass multiple
-
-	--verbose, -v: verbose
-
-	URL: The site to spider.
+	--header, -H: In format name:value - can pass multiple.
+      
+    <url>: The site to spider.
 
 "
 	exit 0
@@ -565,6 +547,7 @@ meta = false
 wordlist = true
 meta_temp_dir = "/tmp/"
 keep = false
+words_with_numbers = false
 show_count = false
 auth_type = nil
 auth_user = nil
@@ -587,6 +570,8 @@ begin
 		case opt
 			when '--help'
 				usage
+			when "--with-numbers"
+				words_with_numbers = true
 			when "--count"
 				show_count = true
 			when "--meta-temp-dir"
@@ -624,6 +609,8 @@ begin
 				offsite = true
 			when '--ua'
 				ua = arg
+			when '--debug'
+				debug = true
 			when '--verbose'
 				verbose = true
 			when '--write'
@@ -687,13 +674,13 @@ url = "http://#{url}" if url !~ /^http(s)?:\/\//
 
 # The spider doesn't work properly if there isn't a / on the end
 if url !~ /\/$/
-	# Commented out for Yori
-	#url = "#{url}/"
+	url = "#{url}/"
 end
 
 word_hash = {}
 email_arr = []
 url_stack = Tree.new
+url_stack.debug = debug
 url_stack.max_depth = depth
 usernames = Array.new()
 
@@ -769,6 +756,8 @@ catch :ctrl_c do
 							allow = (a_url_parsed.host == url_parsed.host) && (a_url_parsed.port == url_parsed.port) && (a_url_parsed.scheme == url_parsed.scheme) ? true : false
 
 							puts "Offsite link, not following: #{a_url}" if !allow && verbose
+						else
+							puts "Allowing offsite links" if @debug
 						end
 					end
 				end
@@ -837,6 +826,8 @@ catch :ctrl_c do
 					end
 				else
 					html = resp.body.to_s.force_encoding("UTF-8")
+					# This breaks on this site http://www.spisa.nu/recept/ as the
+					# replace replaces some of the important characters. Needs a fix
 					html.encode!('UTF-16', 'UTF-8', :invalid => :replace, :replace => '')
 					html.encode!('UTF-8', 'UTF-16')
 
@@ -856,7 +847,7 @@ catch :ctrl_c do
 						body += keywords.gsub(/[>"\/']*/, "")
 					end
 
-					puts body	if debug
+					puts body if debug
 
 					# This bit will not normally fire as all JavaScript is stripped out
 					# by the Nokogiri remove a few lines before this.
@@ -947,7 +938,11 @@ catch :ctrl_c do
 
 						if wordlist
 							# Remove any symbols
-							words.gsub!(/[^[:alpha:]]/i, " ")
+							if words_with_numbers then
+								words.gsub!(/[^a-z0-9]/i, " ")
+							else
+								words.gsub!(/[^a-z]/i, " ")
+							end
 
 							# Add to the array
 							words.split(" ").each do |word|
